@@ -46,26 +46,44 @@ using namespace Baloo;
 App::App(QObject* parent)
     : QObject(parent)
     , m_notifyNewData(STDIN_FILENO, QSocketNotifier::Read)
-    , m_io(STDIN_FILENO, STDOUT_FILENO)
-    , m_tr(0)
+    , m_input()
+    , m_inputStream(&m_input)
+    , m_outputStream(stdout)
+    , m_tr(nullptr)
 {
+    m_input.open(STDIN_FILENO, QIODevice::ReadOnly | QIODevice::Unbuffered );
+    m_inputStream.setByteOrder(QDataStream::BigEndian);
+
     connect(&m_notifyNewData, &QSocketNotifier::activated, this, &App::slotNewInput);
 }
 
 void App::slotNewInput()
 {
+    m_inputStream.startTransaction();
+    m_inputStream >> m_ids;
+
+    if (m_inputStream.status() != QDataStream::Ok) {
+        QCoreApplication::quit();
+        return;
+    }
+
+    if (!m_inputStream.commitTransaction()) {
+        m_ids.clear();
+        return;
+    }
+
     Database *db = globalDatabaseInstance();
-    if (!db->open(Database::OpenDatabase)) {
+    if (!db->open(Database::ReadWriteDatabase)) {
         qCritical() << "Failed to open the database";
         exit(1);
     }
 
-    Q_ASSERT(m_tr == 0);
+    Q_ASSERT(m_tr == nullptr);
 
     m_tr = new Transaction(db, Transaction::ReadWrite);
     // FIXME: The transaction is open for way too long. We should just open it for when we're
     //        committing the data not during the extraction.
-    m_io.newBatch();
+
     QTimer::singleShot(0, this, &App::processNextFile);
 
     /**
@@ -77,21 +95,21 @@ void App::slotNewInput()
 
 void App::processNextFile()
 {
-    if (!m_io.atEnd()) {
+    if (!m_ids.isEmpty()) {
         int delay = m_idleMonitor.isIdle() ? 0 : 10;
 
-        quint64 id = m_io.nextId();
+        quint64 id = m_ids.takeFirst();
 
         QString url = QFile::decodeName(m_tr->documentUrl(id));
-        if (!QFile::exists(url)) {
+        if (url.isEmpty() || !QFile::exists(url)) {
             m_tr->removeDocument(id);
             QTimer::singleShot(0, this, &App::processNextFile);
             return;
         }
 
-        m_io.writeStartedIndexingUrl(url);
+        m_outputStream << "S " << url << endl;
         index(m_tr, url, id);
-        m_io.writeFinishedIndexingUrl(url);
+        m_outputStream << "F " << url << endl;
         m_updatedFiles << url;
 
         QTimer::singleShot(delay, this, &App::processNextFile);
@@ -99,7 +117,7 @@ void App::processNextFile()
     } else {
         m_tr->commit();
         delete m_tr;
-        m_tr = 0;
+        m_tr = nullptr;
 
         /*
         * TODO we're already sending out each file as we start we can simply send out a done
@@ -120,33 +138,31 @@ void App::processNextFile()
 
         // Enable the SocketNotifier for the next batch
         m_notifyNewData.setEnabled(true);
-        m_io.writeBatchIndexed();
+        m_outputStream << "B" << endl;
     }
 }
 
 void App::index(Transaction* tr, const QString& url, quint64 id)
 {
-    QString mimetype = m_mimeDb.mimeTypeForFile(url, QMimeDatabase::MatchContent).name();
+    QString mimetype = m_mimeDb.mimeTypeForFile(url).name();
+    qCDebug(BALOO) << "Indexing" << id << url << mimetype;
 
-    bool shouldIndex = m_config.shouldBeIndexed(url) && m_config.shouldMimeTypeBeIndexed(mimetype);
-    if (!shouldIndex) {
-        // FIXME: This should never be happening!
+    if (!m_config.shouldBeIndexed(url)) {
+        // This apparently happens when the config has changed after the document
+        // was added to the content indexing db
+        qCWarning(BALOO) << "Found" << url << "in the ContentIndexingDB, although it should be skipped";
         tr->removeDocument(id);
         return;
     }
 
-    //
-    // HACK: We only want to index plain text files which end with a .txt
-    //
-    if (mimetype == QLatin1String("text/plain")) {
-        if (!url.endsWith(QLatin1String(".txt"))) {
-            qCDebug(BALOO) << "text/plain does not end with .txt. Ignoring";
-            tr->removePhaseOne(id);
-            return;
-        }
+    // The initial BasicIndexingJob run has been supplied with the file extension
+    // mimetype only, skip based on the "real" mimetype
+    if (!m_config.shouldMimeTypeBeIndexed(mimetype)) {
+        qCDebug(BALOO) << "Skipping" << url << "- mimetype:" << mimetype;
+        tr->removePhaseOne(id);
+        return;
     }
 
-    //
     // HACK: Also, we're ignoring ttext files which are greater tha 10 Mb as we
     // have trouble processing them
     //
@@ -160,7 +176,7 @@ void App::index(Transaction* tr, const QString& url, quint64 id)
 
     // We always run the basic indexing again. This is mostly so that the proper
     // mimetype is set and we get proper type information.
-    // The mimetype fetched in the BasicIQ is fast but not accurate
+    // The mimetype fetched in the BasicIndexingJob is fast but not accurate
     BasicIndexingJob basicIndexer(url, mimetype, BasicIndexingJob::NoLevel);
     basicIndexer.index();
 
